@@ -10,9 +10,13 @@
 ##' pop <- simulateScene()
 ##' distance <- pairDist(pop)
 pairDist <- function(scene) {
-  distMat <- as.matrix(dist(scene[, c("x", "y")]))
-  attr(distMat, "idOrder") <- scene$id
-  attr(distMat, "dimnames") <- NULL
+  if (is.list(scene) & !is.data.frame(scene)) {
+    distMat <- lapply(scene, pairDist)
+  } else {
+    distMat <- as.matrix(dist(scene[, c("x", "y")]))
+    attr(distMat, "idOrder") <- scene$id
+    attr(distMat, "dimnames") <- NULL
+  }
   distMat
 }
 
@@ -25,14 +29,18 @@ pairDist <- function(scene) {
 ##' @return a matrix where the rows are all individuals and the columns are
 ##' their k nearest neighbors
 ##' @export
-##' @seealso \code{\link{knn.dist}}
+##' @seealso \code{\link{knn.dist}}, \code{\link{proximity}}
 ##' @examples
 ##' pop <- simulateScene(10)
 ##' kNearNeighbors(pop, 3)
 kNearNeighbors <- function(scene, k) {
-  knnMatrix <- FNN::knn.dist(scene[c("x", "y")], k = k, algorithm = "brute")
-  rownames(knnMatrix) <- scene$id
-  colnames(knnMatrix) <- paste("k", 1:k, sep = "")
+  if (is.list(scene) & !is.data.frame(scene)) {
+    knnMatrix <- lapply(scene, kNearNeighbors, k)
+  } else {
+    knnMatrix <- FNN::knn.dist(scene[c("x", "y")], k = k, algorithm = "brute")
+    rownames(knnMatrix) <- scene$id
+    colnames(knnMatrix) <- paste("k", 1:k, sep = "")
+  }
   knnMatrix
 }
 
@@ -40,15 +48,14 @@ kNearNeighbors <- function(scene, k) {
 ##'
 ##' @title Make potentials object--spatial proximity
 ##' @param scene a matingScene object
-##' @param method one of "maxProp", and "maxPropSqrd" see details for
+##' @param method one of "maxProp", "maxPropSqrd", or 'knn.dist'; see details for
 ##' further description
-##' @param proximityFun a function used to calculate proximity. Not yet
-##' implemented
 ##' @param averageType whether to calculate individual and population proximity
 ##' using the mean or median
 ##' @param subject whether you want pair, individual, population, or all.
 ##' Specifying more than one is allowed.
 ##' @param zeroPotDist the distance at which potential should be equal to zero
+##' @param k the number of the nearest neighbor to search, if \code{method} is "knn.dist". Defaults to 6, but must be less than population size.
 ##' @return A potentials object containing one more more of the following, depending the
 ##' input for \code{subject}: \cr
 ##' If \code{subject} is "population" the return list will contain a numeric
@@ -61,18 +68,19 @@ kNearNeighbors <- function(scene, k) {
 ##' @details If \code{method} is "maxProp" then proximity between two
 ##' individuals will be calculated as 1 - distance/max(distance).
 ##' If \code{method} is "maxPropSqrd" then proximity between two
-##' individuals will be calculated as (1 - distance/max(distance))^2.
+##' individuals will be calculated as (1 - distance/max(distance))^2. If \code{method} is
+##' "knn.dist" then the function This uses \code{FNN::knn.dist} to return the Euclidian distance of the kth nearest neighbor.
 ##' @author Danny Hanson
 ##' @examples
 ##' pop <- simulateScene()
 ##' proximity(pop, "maxProp")
-proximity <- function(scene, method, proximityFun = NULL, averageType = "mean",
-                      subject = "all", zeroPotDist = NULL) {
-  method <- match.arg(method, c("maxProp", "maxPropSqrd"))
+proximity <- function(scene, method, averageType = "mean",
+                      subject = "all", zeroPotDist = NULL, k = 6) {
+  method <- match.arg(method, c("maxProp", "maxPropSqrd",'knn.dist'))
   subject <- match.arg(subject, c("all", "pair", "population", "individual"),
                        several.ok = TRUE)
   if (is.list(scene) & !is.data.frame(scene)) {
-    potential <- lapply(scene, proximity, method, proximityFun, averageType, subject)
+    potential <- lapply(scene, proximity, method, averageType, subject, zeroPotDist, k)
   } else {
     n <- nrow(scene)
     distMatrix <- pairDist(scene)
@@ -96,7 +104,7 @@ proximity <- function(scene, method, proximityFun = NULL, averageType = "mean",
 
       distNoDiag <- distMatrix[-seq(1, n^2, n+1)]
       distNoDiag[distNoDiag > zeroPotDist] <- zeroPotDist
-      distNoDiagMat <- matrix(distNoDiag, nrow = n, byrow = T)
+      distNoDiagMat <- matrix(distNoDiag, nrow = n, byrow = TRUE)
       pairProx2 <- 1 - distNoDiagMat/zeroPotDist
 
       indProx <- data.frame(id = scene$id, proximity = -1)
@@ -114,7 +122,7 @@ proximity <- function(scene, method, proximityFun = NULL, averageType = "mean",
 
       distNoDiag <- distMatrix[-seq(1, n^2, n+1)]
       distNoDiag[distNoDiag > zeroPotDist] <- zeroPotDist
-      distNoDiagMat <- matrix(distNoDiag, nrow = n, byrow = T)
+      distNoDiagMat <- matrix(distNoDiag, nrow = n, byrow = TRUE)
       pairProx2 <- (1 - distNoDiagMat/zeroPotDist)^2
 
       indProx <- data.frame(id = scene$id, proximity = -1)
@@ -123,8 +131,12 @@ proximity <- function(scene, method, proximityFun = NULL, averageType = "mean",
       } else if (averageType == "median") {
         indProx$proximity <- row_medians(pairProx2)
       }
-
       popProx <- average(indProx[,2])
+    } else if (method == 'knn.dist'){
+        subject <- c('population','individual')
+        indProx <- data.frame(id = scene$id, knn.dist = -1, k = k)
+        indProx$knn.dist <- kNearNeighbors(scene, k)[,k]
+        popProx <- average(indProx$knn.dist)
     }
 
     # return
